@@ -14,12 +14,13 @@ __author__ = (
     "Mark Harviston <mark.harviston@gmail.com>, "
     "Arve Knudsen <arve.knudsen@gmail.com>",
 )
-__version__ = "0.12.0"
+__version__ = "0.15.0"
 __url__ = "https://github.com/CabbageDevelopment/qasync"
 __license__ = "BSD"
 __all__ = ["QEventLoop", "QThreadExecutor", "asyncSlot", "asyncClose"]
 
 import asyncio
+import contextlib
 import functools
 import importlib
 import itertools
@@ -41,6 +42,7 @@ if qtapi_env:
         "pyqt5": "PyQt5",
         "pyqt": "PyQt4",
         "pyqt4": "PyQt4",
+        "pyside6": "PySide6",
         "pyside2": "PySide2",
         "pyside": "PySide",
     }
@@ -58,14 +60,14 @@ if qtapi_env:
 
 # If a Qt lib is already imported, use that
 if not QtModule:
-    for QtModuleName in ("PyQt5", "PySide2"):
+    for QtModuleName in ("PyQt5", "PySide2", "PySide6"):
         if QtModuleName in sys.modules:
             QtModule = sys.modules[QtModuleName]
             break
 
 # Try importing qt libs
 if not QtModule:
-    for QtModuleName in ("PyQt5", "PySide2"):
+    for QtModuleName in ("PyQt5", "PySide2", "PySide6"):
         try:
             QtModule = importlib.import_module(QtModuleName)
         except ImportError:
@@ -80,17 +82,25 @@ logger.info("Using Qt Implementation: {}".format(QtModuleName))
 
 QtCore = importlib.import_module(QtModuleName + ".QtCore", package=QtModuleName)
 QtGui = importlib.import_module(QtModuleName + ".QtGui", package=QtModuleName)
+
 if QtModuleName == "PyQt5":
     from PyQt5 import QtWidgets
     from PyQt5.QtCore import pyqtSlot as Slot
 
     QApplication = QtWidgets.QApplication
+
 elif QtModuleName == "PySide2":
     from PySide2 import QtWidgets
     from PySide2.QtCore import Slot
 
     QApplication = QtWidgets.QApplication
 
+elif QtModuleName == "PySide6":
+    from PySide6 import QtWidgets
+    from PySide6.QtCore import Slot
+
+    QApplication = QtWidgets.QApplication
+
 from ._common import with_logger  # noqa
 
 
@@ -235,7 +245,7 @@ class _SimpleTimer(QtCore.QObject):
         self._stopped = False
 
     def add_callback(self, handle, delay=0):
-        timerid = self.startTimer(delay * 1000)
+        timerid = self.startTimer(int(delay * 1000))
         self._logger.debug("Registering timer id {0}".format(timerid))
         assert timerid not in self.__callbacks
         self.__callbacks[timerid] = handle
@@ -288,6 +298,10 @@ class _QEventLoop:
     >>> asyncio.set_event_loop(loop)
     >>> with loop:
     ...     loop.run_until_complete(xplusy(2, 2))
+
+    If the event loop shall be used with an existing and already running QApplication
+    it must be specified in the constructor via already_running=True
+    In this case the user is responsible for loop cleanup with stop() and close()
     """
 
     def __init__(self, app=None, set_running_loop=True, already_running=False):
@@ -316,8 +330,17 @@ class _QEventLoop:
         if already_running:
             self.__is_running = True
 
+            # it must be ensured that all pre- and
+            # postprocessing for the eventloop is done
+            self._before_run_forever()
+            self.__app.aboutToQuit.connect(self._after_run_forever)
+
     def run_forever(self):
         """Run eventloop forever."""
+
+        if self.__is_running:
+            raise RuntimeError("Event loop already running")
+
         self.__is_running = True
         self._before_run_forever()
 
@@ -332,6 +355,10 @@ class _QEventLoop:
 
     def run_until_complete(self, future):
         """Run until Future is complete."""
+
+        if self.__is_running:
+            raise RuntimeError("Event loop already running")
+
         self._logger.debug("Running {} until complete".format(future))
         future = asyncio.ensure_future(future, loop=self)
 
@@ -523,9 +550,8 @@ class _QEventLoop:
     def __on_notifier_ready(self, notifiers, notifier, fd, callback, args):
         if fd not in notifiers:
             self._logger.warning(
-                "Socket notifier for fd {} is ready, even though it should be disabled, not calling {} and disabling".format(
-                    fd, callback
-                ),
+                f"Socket notifier for fd {fd} is ready, even though it should "
+                f"be disabled, not calling {callback} and disabling"
             )
             notifier.setEnabled(False)
             return
@@ -718,3 +744,30 @@ def asyncSlot(*args):
         return wrapper
 
     return outer_decorator
+
+
+class QEventLoopPolicyMixin:
+    def new_event_loop(self):
+        return QEventLoop(QApplication(sys.argv))
+
+
+class DefaultQEventLoopPolicy(
+    QEventLoopPolicyMixin,
+    asyncio.DefaultEventLoopPolicy,
+):
+    pass
+
+
+@contextlib.contextmanager
+def _set_event_loop_policy(policy):
+    old_policy = asyncio.get_event_loop_policy()
+    asyncio.set_event_loop_policy(policy)
+    try:
+        yield
+    finally:
+        asyncio.set_event_loop_policy(old_policy)
+
+
+def run(*args, **kwargs):
+    with _set_event_loop_policy(DefaultQEventLoopPolicy()):
+        return asyncio.run(*args, **kwargs)
